<?php namespace net\xp_lang\tests\compilation;

/**
 * TestCase for method annotations
 *
 * @see   xp://net.xp_lang.tests.compilation.AnnotationsTest
 */
class MethodAnnotationsTest extends AnnotationsTest {

  /**
   * Test Method::getAnnotations() on compiled type
   *
   */
  #[@test]
  public function noAnnotations() {
    $this->assertEquals([], $this->compile('class %s { void fixture() { } }')->getMethod('fixture')->getAnnotations());
  }

  /**
   * Test Method::getAnnotations() on compiled type
   *
   */
  #[@test]
  public function simpleAnnotation() {
    $this->assertEquals(
      ['experimental' => null], 
      $this->compile('class %s { [@experimental] void fixture() { } }')->getMethod('fixture')->getAnnotations()
    );
  }

  /**
   * Test Method::getAnnotations() on compiled type
   *
   */
  #[@test]
  public function annotationWithDefault() {
    $this->assertEquals(
      ['experimental' => 'beta'], 
      $this->compile('class %s { [@experimental("beta")] void fixture() { } }')->getMethod('fixture')->getAnnotations()
    );
  }

  /**
   * Test Method::getAnnotations() on compiled type
   *
   */
  #[@test]
  public function annotationWithParams() {
    $this->assertEquals(
      ['experimental' => ['stages' => ['beta', 'RC']]], 
      $this->compile('class %s { [@experimental(stages= ["beta", "RC"])] void fixture() { } }')->getMethod('fixture')->getAnnotations()
    );
  }

  /**
   * Test parameter annotation
   *
   * @see   https://github.com/xp-framework/rfc/issues/218
   */
  #[@test]
  public function parameterAnnotation() {
    $type= $this->compile('class %s { [@$conn: inject(name= "db")] void fixture(var $conn) { } }');
    $this->assertEquals(
      ['inject' => ['name' => 'db']], 
      $type->getMethod('fixture')->getParameters()[0]->getAnnotations()
    );
  }
}